@@ -28,6 +28,11 @@ import matplotlib
 import warnings
 warnings.filterwarnings("ignore")
 
+def version():
+    print('Rosahami processor, 17.3.2022')
+    print('Rossendorfer SAXS HAPG mirror.')
+    print('Michal Smid, HZDR')
+    
     
 def run_simulation(p):
     p['source']['continuumAdd']=True
@@ -183,7 +188,9 @@ def evaluate_simulation(p,spectrorrr,quarterring,climmax):
     # print info
     from matplotlib import rc
     ax = fig.add_subplot(133)
-    t=plt.text(-0.1,1,ii,transform=ax.transAxes,FontSize=12,VerticalAlignment='top')
+#    t=plt.text(-0.1,1,ii,transform=ax.transAxes,fontsize=12,VerticalAlignment='top')
+#    t=plt.text(-0.1,1,ii,transform=ax.transAxes,VerticalAlignment='top')
+ #   t=plt.text(-0.1,1,ii)
     ax.axis('off')
     ax.get_xaxis().set_visible(False)
     ax.axes.get_yaxis().set_visible(False)
@@ -504,7 +511,6 @@ def unwrap_map_pictures():
     
     #plt.xlabel('detector X [mm]')
     plt.ylabel('detector Y [mm]')
-    #plt.colorbar()
     plt.plot(ypf*500,detY,'r',linewidth=1)
     
     
@@ -516,7 +522,6 @@ def unwrap_map_pictures():
     prof=alZmap[int(np.shape(alZmap)[0]/2),:]
     plt.plot(detD,prof*500+wY/2,color=[0.5,0.5,0.5]) #plotting the first line
     plt.xlabel('detector X [mm]')
-    #plt.colorbar()
     
     
     plt.plot(detD,zpf*500+wY/2,'r',linewidth=1)
@@ -646,7 +651,7 @@ def do_unwrap(im,p,interpAlX,interpAlY):
          - columns as follows:
          0 alX
          1 alY
-         2 intensity
+         2 plpltintensity
          3 alR
          -remaining columns are free for further use
     
@@ -687,7 +692,7 @@ def do_unwrap(im,p,interpAlX,interpAlY):
 
 
 # %%##############################################################################
-def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF):
+def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF,rawclimmax=20,radialclimmax=1,radialplotymax=20):
     print('-- Draw unwrapping.')
     mradmax=p['draw_max_mrad']
     marksize=0.5    
@@ -709,7 +714,7 @@ def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF):
         plt.title('raw data [px]')
         plt.text(20,20,p['name'],color='white')      
         plt.colorbar()
-        plt.clim(0,20)
+        plt.clim(0,rawclimmax)
     
     # %  
         plt.subplot(232)   #unwrapped data
@@ -746,7 +751,7 @@ def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF):
         points2[2,points[1,:]<0] = np.nan
         plt.plot(points2[3,:]*1e3,points2[2,:],'r.',markersize=0.5)
             #zoom
-        plt.ylim(0.2,20)
+        plt.ylim(0.2,radialplotymax)
         plt.xlim(0,mradmax)
         
         plt.semilogy(alRange*1e3,sigup,linewidth=3,color=[0,0,0.5],label='down')
@@ -767,7 +772,7 @@ def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF):
         plt.subplot(234)   # spherical signal
         plt.title('radially averaged signal')        
         cl1=-0.5
-        cl2=1
+        cl2=radialclimmax
         selleft=points[0,:]<0
         plt.scatter(points[0,selleft]*1e3,points[1,selleft]*1e3,c=np.log10(points[6,selleft]),s=marksize)
         plt.clim(cl1,cl2)
@@ -1000,6 +1005,7 @@ def calculate_peaks(p,areas,points,flatfield):
         numpointsflat=np.sum(selF) #number of points in peak region
         m1=np.nanmean(points[2,sel]) #keV/px in peak region
         m1b=np.nanmean(points[2,selB]) #average value of background, keV/px in bck region
+        backdev=np.std(points[2,selB])# standrad deviation of background
         m1c=(m1-m1b)*numpoints #integrated intensity, backgound subtracted; [keV above background]
         m1c=m1c/photon_energy*p['data_numframes']  #[ph. in peak per all shots]
         m2=np.nanmean(flatfield[9,selF]) #mean of flatfield [keV/px]
@@ -1008,12 +1014,13 @@ def calculate_peaks(p,areas,points,flatfield):
         m2e=3*np.sqrt(m2)
         #m1be=3*np.sqrt(m1b/photon_energy*numpoints*numframes) error of background
         #           0  1    2  3  4    5    6  
-        peaks[i,:]=(m1,m1b,m1c,m2,m1ce,m2e, 0)
+        peaks[i,:]=(m1,m1b,m1c,m2,m1ce,m2e, backdev)
         #0  
     peak_intensity=peaks[:,2]/peaks[:,3]
     #2/3   , errors are in dE(2)=4, dE(3)=5
     #I = X/Y, dI = |I| *( (dX/X)^2 + (dY/Y)^2     )*0.5
-    peak_intensity_error_rel= ((peaks[:,4]/peaks[:,2])**2 + (peaks[:,5]/peaks[:,3])**2)**0.5
+    #needto be checked::
+    peak_intensity_error_rel= ((peaks[:,4]/peaks[:,2])**2 + (peaks[:,5]/peaks[:,3])**2 +( peaks[:,6]**2/peaks[:,2]) )**0.5
     peak_intensity_error=np.multiply(np.abs(peak_intensity),peak_intensity_error_rel)
     peaks[:,0]=peak_intensity
     peaks[:,1]=peak_intensity_error
@@ -1021,7 +1028,7 @@ def calculate_peaks(p,areas,points,flatfield):
     return peaks
 
 
-def draw_grating_figs(points,flatfield,peaks,areas,p):
+def draw_grating_figs(points,flatfield,peaks,peak_indexes,areas,p,xlim=8,ylim=1):
     print('-- Drawing grating figs.')
     numpeaks=np.shape(areas)[0]
     cb=np.array((0.8,0.6,0.4))
@@ -1030,18 +1037,19 @@ def draw_grating_figs(points,flatfield,peaks,areas,p):
     ny=points[1,:]*1e3
     ms=45 #marker size
     if 1:
-        fig,(ax1,ax2) = plt.subplots(figsize=(12,4),nrows=2,sharex=True)  
+        fig,(ax1,ax2) = plt.subplots(figsize=(16,4),nrows=2,sharex=True)  
         plt.axes(ax1)
     
         plt.scatter(nx,ny,c=np.log10(points[2,:])+1.8,s=ms,cmap=rofl.cmap())
-        plt.xlim(-8,8)
-        plt.ylim(-1,1)
-        plt.plot([-10,10],[0,0],'w',linewidth=0.3)
+        plt.xlim(-xlim,xlim)
+        plt.ylim(-ylim,ylim)
+#        plt.plot([-10,10],[0,0],'w',linewidth=0.3)
         #plt.xlabel('alpha X [mrad]')
         #plt.ylabel('alpha Y [mrad]')
         plt.title('a) Scattering on grating (logarithmic colorscale)')
-        plt.text(-7.8,0.6,"photons in peak:",fontsize=10,color=cb*1.2)
-    
+#        plt.text(-7.8,0.6,"photons in peak:",fontsize=10,color=cb*1.2)
+        #plt.colorbar()
+        plt.clim(0,5)
     
         for i in np.arange(numpeaks):
             a=areas[i,:]
@@ -1050,7 +1058,8 @@ def draw_grating_figs(points,flatfield,peaks,areas,p):
             b=expand(a,p['expand_radius'])
             plt.plot((b[0],b[0],b[1],b[1],b[0]),(b[2],b[3],b[3],b[2],b[2]),color=cb,linewidth=lwb)
             if ~np.isnan(peaks[i,2]):
-                plt.text(b[0],0.6,"{:2.0f}".format(peaks[i,2]),fontsize=10,color=cb*1.2)
+                #plt.text(b[0],0.6,"{:2.0f}".format(peaks[i,2]),fontsize=10,color=cb*1.2)
+                plt.text(b[0],0.6,"{:d}".format(peak_indexes[i]),fontsize=10,color=cb*1.2)
     
     # Draw the flatfield
         plt.axes(ax2)
@@ -1062,31 +1071,31 @@ def draw_grating_figs(points,flatfield,peaks,areas,p):
  #       flatintensity[f_upper]=flatintensity[f_upper]/upmean
 #        flatintensity[f_lower]=flatintensity[f_lower]/downmean
         ax2.scatter(nx[:],ny[:],c=flatintensity,s=ms,cmap=rofl.cmap())
-        plt.xlim(-8,8)
-        plt.ylim(-1,1)
+        plt.xlim(-xlim,xlim)
+        plt.ylim(-ylim,ylim)
         
-        plt.plot([-10,10],[0,0],'w',linewidth=0.3)
+#        plt.plot([-10,10],[0,0],'w',linewidth=0.3)
         plt.xlabel(r'scattering angle 2$\theta$ [mrad]')
         plt.title("b) Flat field (linear colorscale)")
-        plt.clim(0.5,2.)
+        #plt.clim(0.5,2.)
         
         # peak markers & count
         cb=np.array((0.8,0.6,0.4))*1.2
-        plt.text(-7.8,0.5,"photons in flatfield:",fontsize=10,color='w')
+        plt.text(-7.8,0.5,"photons in flatfield:",fontsize=10,color='r')
                     
         for i in np.arange(numpeaks):
             a=getFlatfieldRect(areas[i,:],p['flatfield_radius'])
             drawRect(a,lwb)                                
-            plt.text(a[0],0.25,"{:2.0f}".format(peaks[i,3]),fontsize=10,color='w')
+            plt.text(a[0],0.25,"{:2.0f}".format(peaks[i,3]),fontsize=6,color='r')
     
   
         plt.savefig('rosahami_grating_figure_'+p['dataname']+'.png' , bbox_inches ='tight',dpi=200)
         
-def draw_grating_peaks(peaks,peak_indexes,p):
+def draw_grating_peaks(peaks,peak_indexes,p,peaks_ref):
     peak_intensity=peaks[:,0]
     peak_intensity_error=peaks[:,1]
     
-    plt.figure(figsize=(5,4))
+    plt.figure(figsize=(10,8))
 
     peaknumL=peak_indexes*1.0
     peaknumL[peaknumL>0]=np.nan
@@ -1095,19 +1104,51 @@ def draw_grating_peaks(peaks,peak_indexes,p):
     peaknumR[peaknumR<0]=np.nan
     
     ms=7
-    a=plt.plot(peaknumL,peak_intensity,'o-',label='upper',markersize=ms,linewidth=2,color=rofl.o())
-    b=plt.plot(peaknumR,peak_intensity,'*-',label='lower',markersize=ms,linewidth=2,color=rofl.b())
-    plt.errorbar(peaknumL-0.03,peak_intensity,yerr=peak_intensity_error,fmt='none',color=a[0].get_color(),capsize=12,capthick=2,elinewidth=2)
-    plt.errorbar(peaknumR+0.03,peak_intensity,yerr=peak_intensity_error,fmt='none',color=b[0].get_color(),capsize=12,capthick=2,elinewidth=2)
+    cs=6
+    if np.size(peaks_ref)>0:
+        peak_ri=peaks_ref[:,0]*4
+        peak_rie=peaks_ref[:,1]*4
+        plt.semilogy(peaknumR,peak_ri,'-',label='accumulation',markersize=ms,linewidth=2,color=rofl.g2())
+        plt.errorbar(peaknumR,peak_ri,yerr=peak_rie,fmt='none',color=rofl.g3(),capsize=cs,capthick=2,elinewidth=2)
+
     
+    a=plt.semilogy(peaknumL,peak_intensity,'o-',label='single shot left',markersize=ms,linewidth=2,color=rofl.o())
+    b=plt.semilogy(peaknumR,peak_intensity,'*-',label='single shot right',markersize=ms,linewidth=2,color=rofl.b())
+    plt.errorbar(peaknumL-0.03,peak_intensity,yerr=peak_intensity_error,fmt='none',color=a[0].get_color(),capsize=cs,capthick=2,elinewidth=2)
+    plt.errorbar(peaknumR+0.03,peak_intensity,yerr=peak_intensity_error,fmt='none',color=b[0].get_color(),capsize=cs,capthick=2,elinewidth=2)
+
+    manualminerror=np.exp(np.log(peak_intensity)-np.log(peak_intensity-peak_intensity_error))
+    manualmaxerror=np.exp(np.log(peak_intensity+peak_intensity_error)-np.log(peak_intensity))
+   
+    plt.errorbar(peaknumR+0.03,peak_intensity,yerr=manualminerror,fmt='none',color='b',capsize=cs,capthick=2,elinewidth=1)
+    plt.errorbar(peaknumR+0.03,peak_intensity,yerr=manualmaxerror,fmt='none',color='r',capsize=cs,capthick=2,elinewidth=1)
+    
+    
+        
     plt.legend()
-    plt.xlim(2.3,7.5)
-    plt.ylim(0,1.6)
+#    plt.xlim(2.3,7.5)
+    plt.ylim(1e-1,1e5)
     plt.grid()
+    plt.xticks(np.abs(peak_indexes))
     plt.xlabel('peak number')
     plt.ylabel('intensity [a.u.] ')
+    plt.title('run #100, train #567')
     plt.savefig('rosahami_grating_peaks_'+p['dataname']+'.png' , bbox_inches ='tight',dpi=300)
 
+
+def generate_defs(peak_indexes,spacing,hor_size,vert_size,y_offset,x_off=0):
+    numpeaks=np.size(peak_indexes)
+    defs=np.zeros((numpeaks,4))
+    defs[:,0]=peak_indexes*spacing
+    defs[:,0]=defs[:,0]+x_off
+#    defs[defs[:,0]>0,0]=defs[defs[:,0]>0,0]+x_off
+ #   defs[defs[:,0]<0,0]=defs[defs[:,0]<0,0]-x_off
+    defs[:,1]=y_offset
+    defs[:,2]=hor_size
+    defs[:,3]=vert_size
+    return defs
+
+
 def defs_to_areas(defs):
     areas=np.copy(defs)
     areas[:,0]=defs[:,0]-defs[:,2]
@@ -1174,4 +1215,4 @@ def do_flatfield(points,p):
     #6 radialy smoothed signal
     #7 2nd radialy smoothed signal - (taking into account damage)
     #8 2nd flatfiled - (taking into account damage, and it is radialy slightly smoothed)    
-    return points,alRange,sigup,sigdown,sigupndF,sigdownndF
\ No newline at end of file
+    return points,alRange,sigup,sigdown,sigupndF,sigdownndF
